----OPTIONS----
local shortenNumbers = true
---------------

local _, class, classID = UnitClass("player")
local garType = LE_GARRISON_TYPE_7_0
local pName = UnitName("player")
local pRealm = GetRealmName()
local followerType = LE_FOLLOWER_TYPE_GARRISON_7_0
local classHallName = _G["ORDER_HALL_"..class]
local strformat = string.format
local buln = BreakUpLargeNumbers
local artwb = ArtifactWatchBar
local hasHallAccess, currency, curDB


local bCH = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Broker_ClassHall", {
	type = "data source",
	label = "Class Hall",
	icon = "Interface\\Icons\\ClassIcon_"..class,
	text = "No Hall Access",
	OnClick = function()
			if hasHallAccess then
				if (GarrisonLandingPage and GarrisonLandingPage:IsShown()) then
					HideUIPanel(GarrisonLandingPage);
				else
					ShowGarrisonLandingPage(garType);
				end
			end
		end,
})

local function ShowCurrency(tt)
	local name, amount, texture = GetCurrencyInfo(currency)
	tt:AddLine(name)
	tt:AddLine(strformat("%s |T%s:0|t", buln(amount), texture), 1,1,1)
end

local function ShowClassSpecCategories(tt)
	tt:AddLine(FOLLOWERLIST_LABEL_TROOPS)
	local classSpecCategories = C_Garrison.GetClassSpecCategoryInfo(followerType)
	if #classSpecCategories > 0 then
		for i, cat in ipairs(classSpecCategories) do
			tt:AddDoubleLine(cat.name, strformat("%d/%d", cat.count, cat.limit), 1,1,1,1,1,1)
		end
	else
		tt:AddLine(NONE, 1, 1, 1)
	end
end

local function ShowMissions(tt)
	local complete = #C_Garrison.GetCompleteMissions(followerType)
	local onMission = C_Garrison.GetInProgressMissions(followerType) and #C_Garrison.GetInProgressMissions(followerType) or 0	--all the missions we're working on
	local inProgress = onMission - complete	--let's not include the complete # in our in progress #
	local available = C_Garrison.GetAvailableMissions(followerType) and #C_Garrison.GetAvailableMissions(followerType) or 0
	
	local textColor
	if onMission == 0 then
		textColor = "ffff0000"
	elseif complete == onMission then
		textColor = "ff00f200"
	else
		textColor = "ffffffff"
	end
	local missionStatus = strformat("|c%s%d/%d|r", textColor, complete, onMission)
	
	tt:AddDoubleLine(ORDER_HALL_MISSIONS, missionStatus)
	if inProgress == 0 then
		if onMission == 0 then
			tt:AddLine(GARRISON_EMPTY_IN_PROGRESS_LIST, 1, 0, 0)
		elseif complete == onMission then
			tt:AddLine(GARRISON_MISSION_COMPLETE, 0, .95, 0)
		end
		tt:AddLine(strformat(GARRISON_LANDING_AVAILABLE, available), 1, 1, 1)
	else
		if complete > 0 then
			tt:AddLine(strformat(GARRISON_NUM_COMPLETED_MISSIONS, complete), 0, .95, 0)
		else
			tt:AddLine(strformat(GARRISON_NUM_COMPLETED_MISSIONS, complete), 1, 1, 1)
		end
		tt:AddLine(strformat(GARRISON_LANDING_IN_PROGRESS, inProgress), 1, 1, 1)
	end
end

local function ShowResearch(tt)
	tt:AddLine(ORDER_HALL_TALENT_TITLE)
	local researching = false
	local talentIDS = C_Garrison.GetTalentTreeIDsByClassID(garType, classID)
	for i,v in ipairs(talentIDS) do
		--local _,_,tree = C_Garrison.GetTalentTreeInfoForID(garType, v)
		local _,_,tree = C_Garrison.GetTalentTreeInfoForID(v)
		for j, talent in ipairs(tree) do
			if talent.isBeingResearched then
				tt:AddDoubleLine(talent.name, SecondsToTime(talent.researchTimeRemaining), 1, 1, 1, 1, 1, 1)
				researching = true
				--break
			end
		end
	end
	if not researching then
		tt:AddLine(NONE, 1,1,1)
	end
end

local numMap = {[3] = "k", [6] = "m", [9] = "b", [12] = "t"}
local function FormatNumber(num)
	if shortenNumbers and num > 1000 then
		local multiplier, divideBy
		for i = 3, 12, 3 do
			if num > 10^i then
				multiplier = numMap[i]
				divideBy = 10^i
			end
		end
		return strformat("%.2f%s", num/divideBy, multiplier)
	else
		return buln(num)
	end
end

local function ShowArtifactPower(tt)
	tt:AddLine(ARTIFACT_POWER)
	--tt:AddLine(strformat(ARTIFACT_POWER_TOOLTIP_TITLE, buln(artwb.totalXP), buln(artwb.xp), buln(artwb.xpForNextPoint)), 1, 1, 1)
	tt:AddLine(strformat(ARTIFACT_POWER_TOOLTIP_TITLE, FormatNumber(artwb.totalXP), FormatNumber(artwb.xp), FormatNumber(artwb.xpForNextPoint)), 1, 1, 1)
	if artwb.numPointsAvailableToSpend > 0 then
		tt:AddLine(strformat(ARTIFACT_POWER_TOOLTIP_BODY, artwb.numPointsAvailableToSpend), 0, .95, 0, true)
	end
end

function bCH.OnTooltipShow(self)
	self:AddLine("Broker_ClassHall",1,1,1)
	self:AddLine(" ")
	if hasHallAccess then
		ShowCurrency(self)
		self:AddLine(" ")
		ShowMissions(self)
		self:AddLine(" ")
		ShowClassSpecCategories(self)
		self:AddLine(" ")
		ShowResearch(self)
		self:AddLine(" ")
		ShowArtifactPower(self)
		self:AddLine(" ")
		self:AddLine(MINIMAP_ORDER_HALL_LANDING_PAGE_TOOLTIP)
	else
		self:AddLine("No Hall Access")
	end
end

local override
local function OverrideCommandBar()
	if OrderHallCommandBar and not override then
		OrderHallCommandBar:Hide()	--just in case
		OrderHallCommandBar.Show = OrderHallCommandBar.Hide
		OrderHallCommandBar:HookScript("OnMouseUp", function(self, btn)	--just in case
				if btn == "RightButton" then
					self:Hide()
				end
			end)
		override = true
	end
end

local f = CreateFrame("Frame")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("GARRISON_UPDATE")
f:RegisterEvent("ADDON_LOADED")

local function RegisterEvents()
	f:RegisterUnitEvent("UNIT_PHASE", "player")
	f:RegisterEvent("GARRISON_TALENT_COMPLETE")
	f:RegisterEvent("GARRISON_TALENT_UPDATE")
	f:RegisterEvent("PLAYER_ENTERING_WORLD")
end

f:SetScript("OnEvent", function(self, event, arg1)
	if event == "PLAYER_LOGIN" then
		currency = C_Garrison.GetCurrencyTypes(garType)
	elseif event == "ADDON_LOADED" and arg1 == "Blizzard_OrderHallUI" then
		OverrideCommandBar()
	elseif event == "PLAYER_ENTERING_WORLD" then
		C_Garrison.RequestClassSpecCategoryInfo(followerType)
	elseif event == "GARRISON_UPDATE" then
		if not hasHallAccess then
			hasHallAccess = C_Garrison.GetGarrisonInfo(garType)
			if hasHallAccess then
				bCH.text = classHallName
				C_Garrison.RequestClassSpecCategoryInfo(followerType)
				RegisterEvents()
			end
		end	
	elseif event == "UNIT_PHASE" or event == "GARRISON_TALENT_COMPLETE" or event == "GARRISON_TALENT_UPDATE" then
		C_Garrison.RequestClassSpecCategoryInfo(followerType)
	end
end)
